@@ -3,7 +3,7 @@ from __future__ import annotations
 import sys
 from contextlib import suppress
 from functools import wraps
-from inspect import Signature
+from types import LambdaType
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -19,11 +19,18 @@ from typing import (
     Protocol,
     Self,
     Sequence,
+    TypeVar,
     cast,
     overload,
 )
 
-from typing_extensions import TypeVar, deprecated
+if TYPE_CHECKING:
+    from inspect import Signature
+
+if sys.version_info < (3, 13):
+    from typing_extensions import TypeVar, deprecated
+else:
+    from warnings import deprecated
 
 from .builtins import KwargsT
 
@@ -107,9 +114,13 @@ class _InjectSelfMeta(type):
         """
         Recursively flatten and yield all nested inject_self metaclass relationships.
 
-        :param d:   Optional inner dictionary representing the hierarchy level.
-        :yield:     Each `_InjectSelfMeta` subclass found in the hierarchy.
+        Args:
+            d: Optional inner dictionary representing the hierarchy level.
+
+        Yields:
+            Each `_InjectSelfMeta` subclass found in the hierarchy.
         """
+
         if d is None:
             d = _inject_self_cls[cls]
 
@@ -181,9 +192,10 @@ class _InjectSelfBase(Generic[_T_co, _P, _R_co]):
         """
         Initialize the inject_self descriptor.
 
-        :param function:    The function or method to wrap.
-        :param *args:       Positional arguments to pass when instantiating the target class.
-        :param **kwargs:    Keyword arguments to pass when instantiating the target class.
+        Args:
+            function: The function or method to wrap.
+            *args: Positional arguments to pass when instantiating the target class.
+            **kwargs: Keyword arguments to pass when instantiating the target class.
         """
         self._function = function
 
@@ -192,24 +204,41 @@ class _InjectSelfBase(Generic[_T_co, _P, _R_co]):
         self.args = args
         self.kwargs = kwargs
 
-    def __get__(self, instance: Any | None, owner: type | None = None) -> _InjectedSelfFunc[_T_co, _P, _R_co]:  # pyright: ignore[reportGeneralTypeIssues]
+    @overload
+    def __get__(self, instance: None, owner: type) -> _InjectedSelfFunc[_T_co, _P, _R_co]: ...  # pyright: ignore[reportGeneralTypeIssues]
+    @overload
+    def __get__(self, instance: Any, owner: type | None = None) -> Callable[_P, _R_co]: ...  # pyright: ignore[reportGeneralTypeIssues]
+    def __get__(self, instance: Any | None, owner: type | None = None) -> Any:
         """
         Return a wrapped callable that automatically injects an instance as the first argument when called.
         """
-        if owner is None:
-            owner = type(instance)
 
         @wraps(self._function)
         def _wrapper(*args: Any, **kwargs: Any) -> _R_co:
             """
             Call wrapper that performs the actual injection of `self`.
             """
-            if args and isinstance((first_arg := args[0]), (owner, type(owner))):
+            nonlocal instance, owner
+
+            if instance is None and args:
                 # Instance or class explicitly provided as first argument
-                obj = first_arg if isinstance(first_arg, owner) else first_arg()  # type: ignore[operator]
-                args = args[1:]
-            elif instance is None:
+                first_arg = args[0]
+
+                if owner and isinstance(first_arg, owner):
+                    instance, args = first_arg, args[1:]
+                elif isinstance(first_arg, type(owner)):
+                    owner, args = first_arg, args[1:]
+
+            if instance is None:
                 # Accessed via class
+
+                if owner is None:
+                    from ..exceptions import CustomTypeError
+
+                    raise CustomTypeError(
+                        "Cannot determine owner type for class access.", self.__class__, self._function
+                    )
+
                 obj, kwargs = self._handle_class_access(owner, kwargs)
             else:
                 # Accessed via instance
@@ -219,20 +248,23 @@ class _InjectSelfBase(Generic[_T_co, _P, _R_co]):
 
         return _wrapper
 
-    def _handle_class_access[T](self, owner: type[T], kwargs: dict[str, Any]) -> tuple[T, dict[str, Any]]:
+    def _handle_class_access(self, owner: type, kwargs: dict[str, Any]) -> tuple[object, dict[str, Any]]:
         """
         Handle logic when the descriptor is accessed from the class level.
 
-        :param owner:   The class object owning the descriptor.
-        :param kwargs:  Keyword arguments passed to the wrapped function.
-        :return:        A tuple of `(self_object, updated_kwargs)`.
+        Args:
+            owner: The class object owning the descriptor.
+            kwargs: Keyword arguments passed to the wrapped function.
+
+        Returns:
+            A tuple of `(self_object, updated_kwargs)`.
         """
         if isinstance(self, inject_self.cached):
             # Cached instance creation
             try:
                 return _self_objects_cache[owner], kwargs
             except KeyError:
-                return _self_objects_cache.setdefault(owner, owner()), kwargs
+                return _self_objects_cache.setdefault(owner, owner(*self.args, **self.kwargs)), kwargs
 
         if isinstance(self, (inject_self.init_kwargs, inject_self.init_kwargs.clean)):
             # Constructor accepts forwarded kwargs
@@ -251,6 +283,8 @@ class _InjectSelfBase(Generic[_T_co, _P, _R_co]):
                 )
 
             if not self._init_signature:
+                from inspect import Signature
+
                 self._init_signature = Signature.from_callable(owner)
 
             init_kwargs = self.kwargs | {k: kwargs[k] for k in kwargs.keys() & self._init_signature.parameters.keys()}
@@ -266,7 +300,7 @@ class _InjectSelfBase(Generic[_T_co, _P, _R_co]):
         return owner(*self.args, **self.kwargs), kwargs
 
     def __call__(self_, self: _T_co, *args: _P.args, **kwargs: _P.kwargs) -> _R_co:  # type: ignore[misc]  # noqa: N805
-        return self_.__get__(self, type(self))(*args, **kwargs)
+        return self_.__get__(self, None)(*args, **kwargs)
 
     @property
     def __func__(self) -> Callable[Concatenate[_T_co, _P], _R_co]:
@@ -277,6 +311,8 @@ class _InjectSelfBase(Generic[_T_co, _P, _R_co]):
     def __signature__(self) -> Signature:
         """Return (and cache) the signature of the wrapped function."""
         if not self._signature:
+            from inspect import Signature
+
             self._signature = Signature.from_callable(self._function)
         return self._signature
 
@@ -375,8 +411,9 @@ class _InjectKwargsParamsBase(Generic[_T_co, _P, _R_co]):
         """
         Initialize the inject_kwargs_params descriptor.
 
-        :param function:    The target function or method whose parameters will be injected
-                            from the instance's `self.kwargs` mapping.
+        Args:
+            func: The target function or method whose parameters will be injected
+                from the instance's `self.kwargs` mapping.
         """
         self._function = func
         self._signature = None
@@ -452,6 +489,8 @@ class _InjectKwargsParamsBase(Generic[_T_co, _P, _R_co]):
     def __signature__(self) -> Signature:
         """Return (and cache) the signature of the wrapped function."""
         if not self._signature:
+            from inspect import Signature
+
             self._signature = Signature.from_callable(self._function)
         return self._signature
 
@@ -498,25 +537,27 @@ class inject_kwargs_params(_InjectKwargsParamsBase[_T_co, _P, _R_co]):
 
 
 class _ComplexHash[**P, R]:
-    __slots__ = "func"
+    __slots__ = "_func"
 
     def __init__(self, func: Callable[P, R]) -> None:
-        self.func = func
+        self._func = func
 
     def __call__(self, *args: P.args, **kwargs: P.kwargs) -> R:
-        return self.func(*args, **kwargs)
+        return self._func(*args, **kwargs)
 
     @staticmethod
     def hash(*args: Any) -> int:
         """
         Recursively hash every unhashable object in ``*args``.
 
-        :param *args:   Objects to be hashed.
+        Args:
+            *args: Objects to be hashed.
 
-        :return:        Hash of all the combined objects' hashes.
+        Returns:
+            Hash of all the combined objects' hashes.
         """
-
         values = list[str]()
+
         for value in args:
             try:
                 new_hash = hash(value)
@@ -531,29 +572,39 @@ class _ComplexHash[**P, R]:
 @_ComplexHash
 def complex_hash[T](cls: type[T]) -> type[T]:
     """
-    Decorator for classes to add a ``__hash__`` method to them.
+    Class decorator that automatically adds a ``__hash__`` method to the target class.
 
-    Especially useful for NamedTuples.
+    The generated ``__hash__`` method computes a hash value derived from:
+    - the class's name
+    - the values of all attributes listed in its type annotations.
+
+    This is particularly useful for immutable data structures (e.g., NamedTuples or dataclasses).
     """
 
     def __hash__(self: T) -> int:  # noqa: N807
-        return complex_hash.hash(self.__class__.__name__, *(getattr(self, key) for key in self.__annotations__))
+        if sys.version_info >= (3, 14):
+            from annotationlib import get_annotations
+        else:
+            from inspect import get_annotations
 
-    ns = cls.__dict__.copy()
-    ns["__hash__"] = __hash__
+        return complex_hash.hash(self.__class__.__name__, *(getattr(self, key) for key in get_annotations(cls)))
 
-    return type(cls.__name__, (cls,), ns)  # pyright: ignore[reportReturnType]
+    setattr(cls, __hash__.__name__, __hash__)
+
+    return cls
 
 
 def get_subclasses[T](family: type[T], exclude: Sequence[type[T]] = []) -> list[type[T]]:
     """
     Get all subclasses of a given type.
 
-    :param family:  "Main" type all other classes inherit from.
-    :param exclude: Excluded types from the yield. Note that they won't be excluded from search.
-                    For examples, subclasses of these excluded classes will be yield.
+    Args:
+        family: "Main" type all other classes inherit from.
+        exclude: Excluded types from the yield. Note that they won't be excluded from search. For examples, subclasses
+            of these excluded classes will be yield.
 
-    :return:        List of all subclasses of "family".
+    Returns:
+        List of all subclasses of "family".
     """
 
     def _subclasses(cls: type[T]) -> Iterator[type[T]]:
@@ -668,8 +719,9 @@ class classproperty(classproperty_base[_T, _R_co, _T_Any]):
             """
             Clear cached properties of an type instance.
 
-            :param type_:   The type whose cache should be cleared.
-            :param names:   Specific property names to clear. If None, all cached properties are cleared.
+            Args:
+                type_: The type whose cache should be cleared.
+                names: Specific property names to clear. If None, all cached properties are cleared.
             """
             if names is None:
                 with suppress(AttributeError):
@@ -720,10 +772,19 @@ class cachedproperty(property, Generic[_R_co, _T_Any]):
 
     if sys.version_info < (3, 13):
 
-        def __init__(self, fget: Any, fset: Any | None = None, fdel: Any | None = None, doc: str | None = None) -> None:
-            self.__name__ = fget.__name__
+        def __init__(
+            self,
+            fget: Callable[[Any], _R_co],
+            fset: Callable[[Any, _T_Any], None] | None = None,
+            fdel: Callable[[Any], None] | None = None,
+            doc: str | None = None,
+        ) -> None:
+            self.__name__ = fget.__name__ + f"_{id(fget)}" if isinstance(fget, LambdaType) else fget.__name__
             super().__init__(fget, fset, fdel, doc)
 
+        def __set_name__(self, owner: object, name: str) -> None:
+            self.__name__ = name
+
     @overload
     def __get__(self, instance: None, owner: type | None = None) -> Self: ...
 
@@ -758,8 +819,9 @@ class cachedproperty(property, Generic[_R_co, _T_Any]):
         """
         Clear cached properties of an object instance.
 
-        :param obj:   The object whose cache should be cleared.
-        :param names: Specific property names to clear. If None, all cached properties are cleared.
+        Args:
+            obj: The object whose cache should be cleared.
+            names: Specific property names to clear. If None, all cached properties are cleared.
         """
         if names is None:
             obj.__dict__.get(cls.cache_key, {}).clear()
@@ -778,9 +840,10 @@ class cachedproperty(property, Generic[_R_co, _T_Any]):
         """
         Update cached property of an object instance.
 
-        :param obj:   The object whose cache should be updated.
-        :param names: Property name to update.
-        :param value: The value to assign.
+        Args:
+            obj: The object whose cache should be updated.
+            name: Property name to update.
+            value: The value to assign.
         """
         obj.__dict__.setdefault(cls.cache_key, {})[name] = value
 
@@ -794,22 +857,33 @@ class KwargsNotNone(KwargsT):
 
 
 class SingletonMeta(type):
-    _instances: ClassVar[dict[type[Any], Any]] = {}
+    _instances: ClassVar[dict[SingletonMeta, Any]] = {}
     _singleton_init: bool
 
     def __new__[MetaSelf: SingletonMeta](
-        mcls: type[MetaSelf], name: str, bases: tuple[type, ...], namespace: dict[str, Any], **kwargs: Any
+        mcls: type[MetaSelf],
+        name: str,
+        bases: tuple[type, ...],
+        namespace: dict[str, Any],
+        *,
+        init: bool = False,
+        **kwargs: Any,
     ) -> MetaSelf:
-        namespace["_singleton_init"] = kwargs.pop("init", False)
-        return super().__new__(mcls, name, bases, namespace, **kwargs)
+        cls = super().__new__(mcls, name, bases, namespace, **kwargs)
+        cls._singleton_init = init
+        return cls
 
-    def __call__(cls, *args: Any, **kwargs: Any) -> SingletonMeta:
-        if cls not in cls._instances:
-            cls._instances[cls] = super(SingletonMeta, cls).__call__(*args, **kwargs)
-        elif cls._singleton_init:
-            cls._instances[cls].__init__(*args, **kwargs)
+    if not TYPE_CHECKING:
 
-        return cls._instances[cls]
+        def __call__(cls, *args: Any, **kwargs: Any) -> Any:
+            if cls not in cls._instances:
+                cls._instances[cls] = obj = super().__call__(*args, **kwargs)
+                return obj
+
+            if cls._singleton_init:
+                cls._instances[cls].__init__(*args, **kwargs)
+
+            return cls._instances[cls]
 
 
 class Singleton(metaclass=SingletonMeta):
